#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Load, clean, and do some basic data exploration on the data

- load tweets (previously concatenated with cat_tweets) into a single DataFrame
- drop columns with mostly null values
- drop rows with mostly null values
- use DataFrame.describe to get basic stats on each column
- use pandas_profiler.ProfileReport to produce histograms, etc in HTML
"""
from __future__ import division, print_function, absolute_import
# from past.builtins import basestring

import os

import pandas as pd
import pandas_profiling


# you really want to be efficient about RAM, so user iter and itertools
# from itertools import izip
from twip.constant import DATA_PATH

from pug.nlp.util import dict2obj

# the round-trip to disk cleans up encoding issues so encoding option no longer needs to be specified and gzip 
df = pd.read_csv(os.path.join(DATA_PATH, 'cleaned_tweets.csv.gz'), index_col='id', compression='gzip',
                  quotechar='"', quoting=pd.io.common.csv.QUOTE_NONNUMERIC, low_memory=False)


print('df.describe() stats:')
desc = df.describe()
for col, stats in desc.T.iterrows():
    print('')
    print('{} ({})'.format(col, df[col].dtype if isinstance(df[col], pd.Series) else type(df[col])))
    print(stats)

html = pandas_profiling.to_html(df.head(3), desc)
open('report.html', 'w').write(html)

# this is redundant with stats above and takes way longer than it should (30 minutes?)
# print('Column, Count, Min, Mean, Max:')
# for k, c, colmin, colmean, colmax in izip(df.columns, df.count().T, df.min().T, df.mean().T, df.max().T):
#     print('{:40s}\t{}\t{}\t{}\t{}'.format(k, c, colmin, colmean, colmax))

# this takes a few minutes
print('Trying to compute a ProfileReport, including correlation between columns, skew etc')
# pandas_profiling.ProfileReport raises Tkinter exceptions before it can produce any output,
#  at least describe produces a dataframe of stats
report = dict2obj(pandas_profiling.describe(df))
print(report['table'])

print('')
for col, stats in report['variables'].iterrows():
    print('')
    print(col)
    # print('{} ({})'.format(col, df[col].dtype if isinstance(df[col], pd.Series) else type(df[col])))
    print(stats)

print('')
for col, stats in report['freq'].iteritems():
    print('')
    print(stats)




#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Load cleaned tweets and add additional columns for NLP features

- Counter dict of words (sparse word vector)
- Freq dict (sparse TFIDF)
- Num words
- Num characters
"""
from __future__ import division, print_function, absolute_import
# from past.builtins import basestring

import os
from collections import Counter

import pandas as pd
import progressbar

# you really want to be efficient about RAM, so user iter and itertools
# from itertools import izip
from twip.constant import DATA_PATH
from pug.nlp.segmentation import Tokenizer

np = pd.np


segment_words = Tokenizer(ngrams=1, lower=True)


def count_words(df, num_tweets=1000000, verbosity=1):
    if verbosity > 0:
        pbar_i = 0
        pbar = progressbar.ProgressBar(maxval=df.shape[0])
        pbar.start()

    counts = pd.DataFrame(columns=df.index)
    stats = []
    for twid, row in df.iterrows():
        if verbosity:
            pbar_i += 1
            pbar.update(pbar_i)
        if pbar_i > num_tweets:
            break
        text = row.text if isinstance(row.text, basestring) and row.text and not isinstance(row.text, np.nan) else ''
        counts[twid] = pd.Series(Counter(segment_words(text)))
        stats += [[counts[twid].sum(), np.sum(np.array(counts[twid] > 0)), len(text)]]
    stats = pd.DataFrame(stats, index=df.index, columns=['text_num_words', 'text_num_unique', 'text_len'])
    df = pd.concat([df, stats], axis=1)
    if verbosity:
        pbar.finish()
    return df, counts


def run():
    print('Loading previously cleaned tweets...')
    # the round-trip to disk cleans up encoding issues so encoding option no longer needs to be specified and gzip 
    df = pd.read_csv(os.path.join(DATA_PATH, 'cleaned_tweets.csv.gz'), index_col='id', compression='gzip',
                      quotechar='"', quoting=pd.io.common.csv.QUOTE_NONNUMERIC, low_memory=False)
    print('Found {} tweets.'.format(len(df)))
    df, counts = count_words(df)
    df.to_csv(os.path.join(DATA_PATH, 'counted_tweets.csv.gz'), compression='gzip',
              quotechar='"', quoting=pd.io.common.csv.QUOTE_NONNUMERIC)
    counts = counts.T  # word vectors in rows instead of columns
    counts.to_csv(os.path.join(DATA_PATH, 'tweet_word_vectors.csv.gz'), compression='gzip',
                    quotechar='"', quoting=pd.io.common.csv.QUOTE_NONNUMERIC)



# 1. copy this file to settings_secret.py
# 2. make sure the settings_secret.py file is ignored (should be listed in .gitignore)
# 3. EITHER...
#   a) add your TWITTER secrets directly to this file in the `except:` block below
#    **OR**
#   b) set your environment variables to contain your keys (see below for env var names)
# 4. `twyth = Twython(settings_secret.TWITTER_API_KEY, settings_secret.TWITTER_API_SECRET)`

# TWITTER_API_KEY and _SECRET are required to download tweets
try:
    TWITTER_API_KEY = os.getenv('TWITTER_API_KEY')
    TWITTER_API_SECRET = os.getenv('TWITTER_API_SECRET')
except:
    TWITTER_API_KEY = '25AlphaNumericCharactersX'
    TWITTER_API_SECRET = '50AlphaNumericCharactersX50AlphaNumericCharactersX'

# Optional account identifying information that doesn't need to be protected
TWITTER_API_OWNER = 'hobsonlane'  # the app owner's @username
TWITTER_API_OWNER_ID = 59275999   # see the app

# This is how to use these keys to instantiate a Twython connection to the twitter API
# twyth = Twython(TWITTER_API_KEY, TWITTER_API_SECRET)



from __future__ import division, print_function, absolute_import
# `pip install future` for universal python2/3
from past.builtins import basestring

import os
import json
from zipfile import ZipFile

DATA_PATH = os.path.join('..', 'data')


def get_nested(obj, path):
    if isinstance(path, basestring):
        path = [path]
    if path:
        return get_nested(obj.get(path[0], {}), path[1:])
    else:
        return obj


def load_tweets(filename='tweets.zip'):
    r"""Extract the cached tweets "database" if necessary and load + parse the json.

    >>> js = load_tweets()
    >>> len(js)
    8000
    >>> js[0].keys()
    [u'contributors',
     u'truncated',
     u'text',
     u'is_quote_status',
     u'in_reply_to_status_id',
     u'id',
     u'favorite_count',
     u'source',
     u'retweeted',
     u'coordinates',
     u'entities',
     u'in_reply_to_screen_name',
     u'id_str',
     u'retweet_count',
     u'in_reply_to_user_id',
     u'favorited',
     u'user',
     u'geo',
     u'in_reply_to_user_id_str',
     u'possibly_sensitive',
     u'lang',
     u'created_at',
     u'in_reply_to_status_id_str',
     u'place',
     u'metadata']
    >>> print(json.dumps((obj for obj in js if obj['geo'] is not None).next(), indent=4))
    {
        "contributors": null,
        "truncated": false,
        "text": "See our latest #Sacramento, CA #job and click to apply: Python Software Engineer - https://t.co/yimTIlISE0 #IT #Hiring #CareerArc",
        "is_quote_status": false,
        "in_reply_to_status_id": null,
        "id": 674998672136929280,
        "favorite_count": 0,
        "source": "<a href=\"http://www.tweetmyjobs.com\" rel=\"nofollow\">TweetMyJOBS</a>",
        "retweeted": false,
        "coordinates": {
            "type": "Point",
            "coordinates": [
                -121.4399041,
                38.5963157
            ]
        },
        "entities": {
            "symbols": [],
            "user_mentions": [],
            "hashtags": [
                {
                    "indices": [
                        15,
                        26
                    ],
                    "text": "Sacramento"
                },
                {
                    "indices": [
                        31,
                        35
                    ],
                    "text": "job"
                },
                {
                    "indices": [
                        107,
                        110
                    ],
                    "text": "IT"
                },
                {
                    "indices": [
                        111,
                        118
                    ],
                    "text": "Hiring"
                },
                {
                    "indices": [
                        119,
                        129
                    ],
                    "text": "CareerArc"
                }
            ],
            "urls": [
                {
                    "url": "https://t.co/yimTIlISE0",
                    "indices": [
                        83,
                        106
                    ],
                    "expanded_url": "http://bit.ly/1OTNflo",
                    "display_url": "bit.ly/1OTNflo"
                }
            ]
        },
        "in_reply_to_screen_name": null,
        "id_str": "674998672136929280",
        "retweet_count": 0,
        "in_reply_to_user_id": null,
        "favorited": false,
        "user": {
            "follow_request_sent": null,
            "has_extended_profile": false,
            "profile_use_background_image": true,
            "id": 22634351,
            "verified": false,
            "profile_text_color": "000000",
            "profile_image_url_https": "https://pbs.twimg.com/profile_images/670049883869458435/J_Klv-BV_normal.jpg",
            "profile_sidebar_fill_color": "407DB0",
            "is_translator": false,
            "geo_enabled": true,
            "entities": {
                "url": {
                    "urls": [
                        {
                            "url": "https://t.co/DByWt45HZj",
                            "indices": [
                                0,
                                23
                            ],
                            "expanded_url": "http://www.careerarc.com/job-seeker",
                            "display_url": "careerarc.com/job-seeker"
                        }
                    ]
                },
                "description": {
                    "urls": []
                }
            },
            "followers_count": 452,
            "protected": false,
            "location": "Sacramento, CA",
            "default_profile_image": false,
            "id_str": "22634351",
            "lang": "en",
            "utc_offset": -18000,
            "statuses_count": 157,
            "description": "Follow this account for geo-targeted Software Dev.
                - General/IT job tweets in Sacramento, CA. Need help? Tweet us at @CareerArc!",
            "friends_count": 326,
            "profile_link_color": "4A913C",
            "profile_image_url": "http://pbs.twimg.com/profile_images/670049883869458435/J_Klv-BV_normal.jpg",
            "notifications": null,
            "profile_background_image_url_https": "https://pbs.twimg.com/profile_background_images/315958568/Twitter-BG_2_bg-image.jpg",
            "profile_background_color": "253956",
            "profile_banner_url": "https://pbs.twimg.com/profile_banners/22634351/1448587317",
            "profile_background_image_url": "http://pbs.twimg.com/profile_background_images/315958568/Twitter-BG_2_bg-image.jpg",
            "name": "TMJ-SAC IT Jobs",
            "is_translation_enabled": false,
            "profile_background_tile": false,
            "favourites_count": 0,
            "screen_name": "tmj_sac_it",
            "url": "https://t.co/DByWt45HZj",
            "created_at": "Tue Mar 03 15:28:22 +0000 2009",
            "contributors_enabled": false,
            "time_zone": "Eastern Time (US & Canada)",
            "profile_sidebar_border_color": "000000",
            "default_profile": false,
            "following": null,
            "listed_count": 36
        },
        "geo": {
            "type": "Point",
            "coordinates": [
                38.5963157,
                -121.4399041
            ]
        },
        "in_reply_to_user_id_str": null,
        "possibly_sensitive": false,
        "lang": "en",
        "created_at": "Thu Dec 10 17:06:38 +0000 2015",
        "in_reply_to_status_id_str": null,
        "place": {
            "country_code": "US",
            "url": "https://api.twitter.com/1.1/geo/id/b71fac2ee9792cbe.json",
            "country": "United States",
            "place_type": "city",
            "bounding_box": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [
                            -121.576613,
                            38.43792
                        ],
                        [
                            -121.362715,
                            38.43792
                        ],
                        [
                            -121.362715,
                            38.685512
                        ],
                        [
                            -121.576613,
                            38.685512
                        ]
                    ]
                ]
            },
            "contained_within": [],
            "full_name": "Sacramento, CA",
            "attributes": {},
            "id": "b71fac2ee9792cbe",
            "name": "Sacramento"
        },
        "metadata": {
            "iso_language_code": "en",
            "result_type": "recent"
        }
    }
    """
    basename, ext = os.path.splitext(filename)
    json_file = basename + '.json'
    json_path = os.path.join(DATA_PATH, json_file)
    zip_path = os.path.join(DATA_PATH, basename + '.zip')
    if not os.path.isfile(json_path):
        zf = ZipFile(zip_path, 'r')
        zf.extract(json_file, DATA_PATH)
    with open(json_path, 'rUb') as f:
        return json.load(f)


def features(filename='tweets.zip'):
    """NotImplemented"""
    raise NotImplemented("Need to implement feature extraction.")
    js = load_tweets(filename)
    index = []
    table = []
    columns = [u'text', u'is_quote_status',
               u'in_reply_to_status_id', u'in_reply_to_user_id',
               u'favorited', u'favorite_count',  u'retweeted', u'retweet_count',
               u'source', u'coordinates',
               (u'user', u'id'), (u'user', u'followers_count'), (u'user', u'location'),
               (u'metadata', u'iso_language_code'),
               u'user',
               u'geo',
               u'in_reply_to_user_id_str',
               u'possibly_sensitive',
               u'lang',
               u'created_at',
               u'in_reply_to_status_id_str',
               u'place',
               ]

    for tweet in js:
        index += [int(tweet['id'])]
        txt += [tweet['text']]





from __future__ import division, print_function, absolute_import
# `pip install future` for universal python2/3
from past.builtins import basestring

import re

# import gensim

from twip.regex import RE_TOKEN, RE_NONWORD
from twip.util import str_strip, str_lower, to_ascii, stringify
from twip.nlp import make_named_stemmer

CRE_TOKEN = re.compile(RE_TOKEN)


class Tokenizer(object):
    """Callable and iterable class that yields substrings split on spaces or other configurable delimitters.

    For both __init__ and __call__, doc is the first arg.
    TODO: All args and functionality of __init__() and __call__() should be the same.

    FIXME: Implement the `nltk.tokenize.TokenizerI` interface
           Is it at all pythonic to make a class callable and iterable?
           Is it pythonic to have to instantiate a TokenizerI instance and then call that instance's `tokenize` method?

    >>> abc = (chr(ord('a') + (i % 26)) for i in xrange(1000))
    >>> tokenize = Tokenizer(ngrams=5)
    >>> ans = list(tokenize(' '.join(abc)))
    >>> ans[:7]
    ['a', 'b', 'c', 'd', 'e', 'f', 'g']
    >>> ans[1000:1005]
    ['a b', 'b c', 'c d', 'd e', 'e f']
    >>> ans[1999:2004]
    ['a b c', 'b c d', 'c d e', 'd e f', 'e f g']
    >>> tokenize = Tokenizer(stem='Porter')
    >>> doc = "Here're some stemmable words provided to you for your stemming pleasure."
    >>> sorted(set(tokenize(doc)) - set(Tokenizer(doc, stem='Lancaster')))
    [u"Here'r", u'pleasur', u'some', u'stemmabl', u'your']
    >>> sorted(set(Tokenizer(doc, stem='WordNet')) - set(Tokenizer(doc, stem='Lancaster')))
    ["Here're", 'pleasure', 'provided', 'some', 'stemmable', 'stemming', 'your']
    """
    __safe_for_unpickling__ = True

    def __init__(self, doc=None, regex=CRE_TOKEN, strip=True, nonwords=False, nonwords_set=None, nonwords_regex=RE_NONWORD,
                 lower=None, stem=None, ngrams=1):
        # specific set of characters to strip
        self.strip_chars = None
        if isinstance(strip, basestring):
            self.strip_chars = strip
            # strip_chars takes care of the stripping config, so no need for strip function anymore
            self.strip = None
        elif strip is True:
            self.strip_chars = '-_*`()"' + '"'
        strip = strip or None
        # strip whitespace, overrides strip() method
        self.strip = strip if callable(strip) else (str_strip if strip else None)
        self.doc = to_ascii(doc)
        self.regex = regex
        if isinstance(self.regex, basestring):
            self.regex = re.compile(self.regex)
        self.nonwords = nonwords  # whether to use the default REGEX for nonwords
        self.nonwords_set = nonwords_set or set()
        self.nonwords_regex = nonwords_regex
        self.lower = lower if callable(lower) else (str_lower if lower else None)
        self.stemmer_name, self.stem = make_named_stemmer(stem)  # stem can be a callable Stemmer instance or just a function
        self.ngrams = ngrams or 1  # ngram degree, numger of ngrams per token
        if isinstance(self.nonwords_regex, basestring):
            self.nonwords_regex = re.compile(self.nonwords_regex)
        elif self.nonwords:
            try:
                self.nonwords_set = set(self.nonwords)
            except TypeError:
                self.nonwords_set = set(['None', 'none', 'and', 'but'])
                # if a set of nonwords has been provided dont use the internal nonwords REGEX?
                self.nonwords = not bool(self.nonwords)

    def __call__(self, doc):
        """Lazily tokenize a new document (tokens aren't generated until the class instance is iterated)

        >>> list(Tokenizer()('new string to parse'))
        ['new', 'string', 'to', 'parse']
        """
        # tokenization doesn't happen until you try to iterate through the Tokenizer instance or class
        self.doc = to_ascii(doc)
        # need to return self so that this will work: Tokenizer()('doc (str) to parse even though default doc is None')
        return self
    # to conform to this part of the nltk.tokenize.TokenizerI interface
    tokenize = __call__

    def __reduce__(self):
        """Unpickling constructor and args so that pickling can be done efficiently without any bound methods, etc"""
        return (Tokenizer, (None, self.regex, self.strip, self.nonwords, self.nonwords_set, self.nonwords_regex,
                self.lower, self.stemmer_name, self.ngrams))

    def span_tokenize(self, s):
        """Identify the tokens using integer offsets `(start_i, end_i)` rather than copying them to a new sequence

        The sequence of tokens (strings) can be generated with

            `s[start_i:end_i] for start_i, end_i in span_tokenize(s)`

        Returns:
          generator of 2-tuples of ints, like ((int, int) for token in s)
        """
        return
        # raise NotImplementedError("span_tokenizer interface not yet implemented, so just suck it up and use RAM to tokenize() ;)")

    def tokenize_sents(self, strings):
        """NTLK.
        Apply ``self.tokenize()`` to each element of ``strings``.  I.e.:
            return [self.tokenize(s) for s in strings]
        :rtype: list(list(str))
        """
        return [self.tokenize(s) for s in strings]

    def span_tokenize_sents(self, strings):
        """
        Apply ``self.span_tokenize()`` to each element of ``strings``.  I.e.:
            return iter((self.span_tokenize(s) for s in strings))
        :rtype: iter(list(tuple(int, int)))
        """
        for s in strings:
            yield list(self.span_tokenize(s))

    def __iter__(self, ngrams=None):
        r"""Generate a sequence of words or tokens, using a re.match iteratively through the str

        TODO:
          - need two different self.lower and lemmatize transforms, 1 before and 1 after nonword detection
          - each of 3 nonword filters on a separate line, setting w=None when nonword "hits"
          - refactor `nonwords` arg/attr to `ignore_stopwords` to be more explicit

        >>> doc = "John D. Rock\n\nObjective: \n\tSeeking a position as Software --Architect-- / _Project Lead_ that can utilize my expertise and"
        >>> doc += " experiences in business application development and proven records in delivering 90's software. "
        >>> doc += "\n\nSummary: \n\tSoftware Architect"
        >>> doc += " who has gone through several full product-delivery life cycles from requirements gathering to deployment / production, and"
        >>> doc += " skilled in all areas of software development from client-side JavaScript to database modeling. With strong experiences in:"
        >>> doc += " \n\tRequirements gathering and analysis."

        The python splitter will produce 2 tokens that are only punctuation ("/")
        >>> len([s for s in doc.split() if s])
        72

        The built-in nonword REGEX ignores all-punctuation words, so there are 2 less here:
        >>> len(list(Tokenizer(doc, strip=False, nonwords=False)))
        70

        In addition, punctuation at the end of tokens is stripped so "D. Rock" doesn't tokenize to "D." but rather "D"
        >>> run_together_tokens = ''.join(list(Tokenizer(doc, strip=False, nonwords=False)))
        >>> '/' in run_together_tokens or ':' in ''.join(run_together_tokens)
        False

        But you can turn off stripping when instantiating the object.
        >>> all(t in Tokenizer(doc, strip=False, nonwords=True) for t in ('D', '_Project', 'Lead_', "90's", "product-delivery"))
        True
        """
        ngrams = ngrams or self.ngrams
        # FIXME: Improve memory efficiency by making this ngram tokenizer an actual generator
        if ngrams > 1:
            original_tokens = list(self.__iter__(ngrams=1))
            for tok in original_tokens:
                yield tok
            for i in range(2, ngrams + 1):
                for tok in list_ngrams(original_tokens, n=i, join=' '):
                    yield tok
        else:
            for w in self.regex.finditer(self.doc):
                if w:
                    w = w.group()
                    w = w if not self.strip_chars else str_strip(w, self.strip_chars)
                    w = w if not self.strip else self.strip(w)
                    w = w if not self.stem else self.stem(w)
                    w = w if not self.lemmatize else self.lemmatize(w)
                    w = w if not self.lower else self.lower(w)
                    # FIXME: nonword check before and after preprossing? (lower, lemmatize, strip, stem)
                    # 1. check if the default nonwords REGEX filter is requested, if so, use it.
                    # 2. check if a customized nonwords REGES filter is provided, if so, use it.
                    # 3. make sure the word isn't in the provided (or empty) set of nonwords
                    if w and (not self.nonwords or not re.match(r'^' + RE_NONWORD + '$', w)) and (
                            not self.nonwords_regex or not self.nonwords_regex.match(w)) and (
                            w not in self.nonwords_set):
                        yield w

    # can these all just be left to default assignments in __init__ or as class methods assigned to global `passthrough()`
    def strip(self, s):
        """Strip punctuation surrounding a token"""
        return s

    def stem(self, s):
        """Find the lexial root of a word, e.g. convert 'running' to 'run'"""
        return s

    def lemmatize(self, s):
        """Find the semantic root of a word, e.g. convert 'was' to 'be'"""
        return s

    def __getstate__(self):
        return self.__dict__

    def __setstate__(self, d):
        self.__dict__.update(d)


def lsi(tokenize=None, iter_dics=None, ram=0.4, num_docs=5000, num_topics=200, num_tokens=1000000,
        num_plots=4, local_path=DATA_PATH, suffix='', ngrams=2,
        verbose=1, show=True, save=True, **kwargs):
    """Train an LSI model on a "corpus" of documnets, like tweets

    Arguments:
      tokenize (callable): Function that returns a sequence of tokens (str) when passed a document (str)
      ram (float):         Portion of RAM to limit the document generator from consuming before StopIteration
      num_docs (int):      Number of documents to limit the document generator to before StopIteration
      num_topics (int):    See gensim.models.LsiModel, default: 200
      chunksize (int):     See gensim.models.LsiModel, default: 10000
      power_iters (int):   See gensim.models.LsiModel, default: None
      extra_samples (int): See gensim.models.LsiModel, default: None

    My process the document generator 3 times (internal tees maintained to "reset" the generator):

    1. "train" Dictionary (Vocab): find set of unique tokens and assigned int IDs)
    2. "train" TfidfModel transform: need global frequencies (by tokenizing docs from document generator) for DF (document freuency) part
    3. "train" LsiModel transform: need to iterate through the word vectors which are generated afresh by retokenizing documents

    TODO: GET RID OF UNUSED ARGS!
          Does the order of documents affect the resulting PCA topics?
          Are topics limited to 10 tokens in gensim?
          Should matches/pairs follow one another?
          Should nonmatching vocab be used at all?

    >>> lsi(ram=0.1, num_docs=2, verbose=0)
    <gensim.models.lsimodel.LsiModel object at ...>
    """
    lsi_kwargs = {'num_topics': num_topics, 'onepass': True}
    lsi_kwargs.update(kwargs)
    if lsi_kwargs.get('onepass', True) and (lsi_kwargs.get('power_iters', None) or lsi_kwargs.get('extra_samples', None)):
        lsi_kwargs['onepass'] = False
    mem0 = psutil.virtual_memory()
    tokenize = tokenize or Tokenizer(ngrams=ngrams)  # , stem='WordNet', lower=True)
    tokenize('Dummy text to get tokenizer to load any data it needs.')
    mem1_tokenizer = psutil.virtual_memory()
    if verbose:
        print('{} tokenizer used {}B'.format(tokenize, friendly(mem1_tokenizer.used - mem0.used)))
    suffix = suffix or 'auto'
    suffix = ('_' * (bool(suffix) and not suffix.startswith('_'))) + suffix
    docgen_orig = (stringify(d) for d in iter_docs)
    docgen_orig, docgen = tee_gen_limit(docgen_orig, limit=num_docs, ram=ram, interval=1000, verbose=verbose)  # 1st generator copy
    mem2_gen = psutil.virtual_memory()
    vocab = Vocab((tokenize(s) for s in docgen), prune_at=num_tokens)
    mem3_vocab = psutil.virtual_memory()
    if verbose:
        print('Built vocabulary of {} unique words and used {}B of RAM. Total RAM used = {}B or {:.3%}'.format(
              friendly(len(vocab)), friendly(mem3_vocab.used - mem2_gen.used), friendly(mem3_vocab.used - mem0.used),
              1. * (mem3_vocab.total - mem3_vocab.available) / mem3_vocab.available))
    if suffix == '_auto':
        suffix = '_' + str(len(vocab)) + '_{}grams'.format(ngrams)
    full_name = 'lsi' + suffix
    if verbose and save:
        print('Will save LsiModel under the name {}'.format(full_name))
    tfidf = TfidfModel(dictionary=vocab)
    mem4_tfidf = psutil.virtual_memory()
    if verbose:
        print('Built tfidf of {} docs and used {}B of RAM. Total RAM used = {}B or {:.4%}'.format(
              friendly(tfidf.num_docs), friendly(mem4_tfidf.used - mem3_vocab.used), friendly(mem4_tfidf.used - mem0.used),
              1. * (mem4_tfidf.total - mem4_tfidf.available) / mem4_tfidf.available))
    docgen_orig, docgen = tee_gen_limit(docgen_orig, limit=num_docs, ram=ram, interval=1000, verbose=verbose)  # 2nd generator copy
    lsi = LsiModel(tfidf[(vocab.doc2bow(tokenize(s)) for s in docgen_orig)], id2word=vocab, **lsi_kwargs)
    mem5_lsi = psutil.virtual_memory()
    if verbose:
        print('Built LSI model: {} docs, {} terms, {} topics. '.format(
            friendly(lsi.docs_processed), friendly(lsi.num_terms), friendly(lsi.num_topics)))
        print('Used {}B of RAM. Total RAM: {}B or {:.4%}'.format(
            friendly(mem5_lsi.used - mem4_tfidf.used), friendly(mem5_lsi.used - mem0.used),
            1. * (mem5_lsi.total - mem5_lsi.available) / mem5_lsi.available))
    # TODO: Consider marshaling the tokenizer and any other user-specified functions
    #   Marshaling stores the bytecode for the tokenizer rather than just its fully-qualified name
    #   This would prevent changes to the tokenizer code from causing previously pickled objects
    #   to behave differently than when they were pickled.
    if save:
        try_preserve(lsi, name=full_name, local_path=local_path, ext='', verbose=verbose)
        # TODO: set lsi.tfidf.vocab=None # already saved in lsi.id2word
        try_preserve(tokenize, name=full_name + '_tokenizer', local_path=local_path, ext='.pickle', verbose=verbose)
    return lsi



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Load previously dumped DataFrame containing normalized lat/lon and plot them
"""
from __future__ import division, print_function, absolute_import

import os
import pandas as pd
from matplotlib import pyplot as plt

from twip.constant import DATA_PATH

# df = pd.io.json.json_normalize(pd.json.load(open('data.json')))
# df.to_csv('data.csv')

geo = pd.read_csv(os.path.join(DATA_PATH, 'geo_tweets.csv'), encoding='utf8', engine='python')
plt.plot(geo.lon, geo.lat, '.')
plt.xlabel('Longitude (deg)')
plt.ylabel('Latitude (deg)')
plt.show()


from mpl_toolkits.basemap import Basemap
import matplotlib.pyplot as plt
import numpy as np
# set up orthographic map projection with
# perspective of satellite looking down at 50N, 100W.
# use low resolution coastlines.
# don't plot features that are smaller than 1000 square km.
globe = Basemap(projection='ortho', lat_0=50, lon_0=-100,
                resolution='l', area_thresh=1000.)
# draw coastlines, country boundaries, fill continents.
globe.drawcoastlines()
globe.drawcountries()
globe.fillcontinents(color='coral')
# draw the edge of the globe projection region (the projection limb)
globe.drawmapboundary()
# draw lat/lon grid lines every 30 degrees.
globe.drawmeridians(np.arange(0, 360, 30))
globe.drawparallels(np.arange(-90, 90, 30))
plt.show()


# lat/lon coordinates of five random cities from scipy example
# lats = [40.02, 32.73, 38.55, 48.25, 17.29]
# lons = [-105.16, -117.16, -77.00, -114.21, -88.10]
# cities=['Boulder, CO','San Diego, CA',
#         'Washington, DC','Whitefish, MT','Belize City, Belize']

# compute the projected coordinates for the points
x, y = globe(df.lon, df.lat)
# plot filled circles at the locations of the cities.
globe.plot(x, y,'b.')

# # plot the names of those tweets/users/etc
# for name, xpt, ypt in zip(names, x, y):
#     plt.text(xpt + 50000, ypt + 50000, name)



from __future__ import division, print_function, absolute_import
# `pip install future` for universal python2/3
from past.builtins import basestring

from time import sleep
import json

from twython import Twython
# from gensim import corpora, models, similarities


class LimittedStream(list):
    """Stream for serializing a sequence of objects so it looks like a list (to json.dump)

    References:
      Vadim Pushtaev: http://stackoverflow.com/a/24033219/623735
    """
    def __init__(self, sequence, limit=1000):
        super(self, LimittedStream).__init__(self)
        self._limit = int(limit)
        self._sequence = int(sequence)

    def __iter__(self):
        """Define a generator function and return it"""
        def generator():
            for i, obj in enumerate(self._sequence):
                if i >= self._limit:
                    break
                yield obj
            raise StopIteration
        return generator

    def __len__(self):
        """Trick json.dump into streaming this container's sequence elements one at a time"""
        return 1


LOCATIONS = {'Portland': (45.52, -122.6819, 20), 'San Jose': (37.3382, -121.8863, 30)}
DISTANCE_UNITS = ('mi', 'km', 'm', 'ft', 'yd')
TWITTER_SEARCH_RATE_LIMIT = 450 / 60. / 15.


def geocode(location, *args, **kwargs):
    """Guess a location lat/lon from the provided string

    Returns:
      str: +##.####,+###.####,### latitude, longitude, distance
           A location and radius in twitter geocode format

    >>> geocode('')
    '45.52,-122.6819,20mi'
    """
    location = LOCATIONS.get(location, location)
    if len(args):
        location = [location] + args
    try:
        llr = location.split(',')
    except:
        llr = [str(s).strip.lower() for s in location]
    llr = [s for s in llr if s]
    for i in range(len(LOCATIONS['Portland'])):
        if len(llr) < i + 1:
            llr += [LOCATIONS['Portland'][i]]
    if str(llr[-1])[-2:] not in DISTANCE_UNITS:
        llr[-1] = str(llr[-1]) + DISTANCE_UNITS[0]
    return ','.join([str(value) for value in llr])


def get_twitter(app_key=None, app_secret=None, search='python', location='', **kwargs):
    """Location may be specified with a string name or latitude, longitude, radius"""
    if not app_key:
        from settings_secret import TWITTER_API_KEY as app_key
    if not app_secret:
        from settings_secret import TWITTER_API_SECRET as app_secret
    twitter = Twython(app_key, app_secret, oauth_version=2)
    return Twython(app_key, access_token=twitter.obtain_access_token())


#TODO: inherit the Twython class and add methods to find keys in files by default
def get_cursor(twitter=None, search='python', location=''):
    if not twitter:
        twitter = get_twitter()
    location = geocode(location)
    return twitter.cursor(twitter.search, q=search)


def limitted_dump(cursor=None, twitter=None, path='tweets.json', limit=450, rate=TWITTER_SEARCH_RATE_LIMIT, indent=-1):
    """Dump a limitted number of json.dump-able objects to the indicated file

    rate (int): Number of queries per 15 minute twitter window
    """
    if not twitter:
        twitter = get_twitter()
    cursor = cursor or 'python'
    if isinstance(cursor, basestring):
        cursor = get_cursor(twitter, search=cursor)
    newline = '\n' if indent is not None else ''
    if indent < 0:
        indent = None
    # TODO: keep track of T0 for the optimal "reset" sleep duration
    with (open(path, 'w') if not isinstance(path, file) else path) as f:
        f.write('[\n')
        for i, obj in enumerate(cursor):
            f.write(json.dumps(obj, indent=indent))
            if i < limit - 1:
                f.write(',' + newline)
            else:
                break
            remaining = int(twitter.get_lastfunction_header('x-rate-limit-remaining'))
            if remaining > 0:
                sleep(1. / rate)
            else:
                sleep(15 * 60)
        f.write('\n]\n')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
This is a skeleton file that can serve as a starting point for a Python
console script. To run this script uncomment the following line in the
entry_points section in setup.cfg:

    console_scripts =
        hello_world = twip.module:function

Then run `python setup.py install` which will install the command `hello_world`
inside your current environment.
Besides console scripts, the header (i.e. until _logger...) of this file can
also be used as template for Python modules.

Note: This skeleton file can be safely removed if not needed!
"""
from __future__ import division, print_function, absolute_import

import argparse
import sys
import logging

from twip import __version__

__author__ = "Hobson Lane"
__copyright__ = "Hobson Lane"
__license__ = "mit"

_logger = logging.getLogger(__name__)


def parse_args(args):
    """
    Parse command line parameters

    :param args: command line parameters as list of strings
    :return: command line parameters as :obj:`argparse.Namespace`
    """
    parser = argparse.ArgumentParser(
        description="Just a Hello World demonstration")
    parser.add_argument(
        '-v',
        '--version',
        action='version',
        version='twip {ver}'.format(ver=__version__))
    return parser.parse_args(args)


def main(args):
    args = parse_args(args)
    print("Hello World!")
    _logger.info("Script ends here")


def run():
    logging.basicConfig(level=logging.INFO, stream=sys.stdout)
    main(sys.argv[1:])


if __name__ == "__main__":
    run()
